@@ -20,8 +20,6 @@ from model_report.utils import base_label, ReportValue, ReportRow
 from model_report.highcharts import HighchartRender
 from model_report.widgets import RangeField
 from model_report.export_pdf import render_to_pdf
-from django.utils.datetime_safe import datetime
-import time
 
 
 try:
@@ -61,7 +59,10 @@ def autodiscover():
                 raise
 
 
-class ReportInstanceManager(object):
+class ReportClassManager(object):
+    """
+    Class to handle registered reports class.
+    """
 
     _register = OrderedDict()
 
@@ -71,24 +72,31 @@ class ReportInstanceManager(object):
     def register(self, slug, rclass):
         if slug in self._register:
             raise ValueError('Slug already exists: %s' % slug)
-        report = rclass()
-        setattr(report, 'slug', slug)
-        self._register[slug] = report
+        setattr(rclass, 'slug', slug)
+        self._register[slug] = rclass
 
     def get_report(self, slug):
+        # return class
         return self._register.get(slug, None)
 
     def get_reports(self):
+        # return clasess
         return self._register.values()
 
 
-reports = ReportInstanceManager()
+reports = ReportClassManager()
 
 
 _cache_class = {}
 
 
 def cache_return(fun):
+    """
+    Usages of this decorator have been removed from the ReportAdmin base class.
+
+    Caching method returns gets in the way of customization at the implementation level
+    now that report instances can be modified based on request data.
+    """
     def wrap(self, *args, **kwargs):
         cache_field = '%s_%s' % (self.__class__.__name__, fun.func_name)
         if cache_field in _cache_class:
@@ -100,28 +108,80 @@ def cache_return(fun):
 
 
 class ReportAdmin(object):
+    """
+    Class to represent a Report.
+    """
 
     fields = []
+    """List of fields or lookup fields for query results to be listed."""
+
     model = None
+    """Primary django model to query."""
+
     list_filter = ()
+    """List of fields or lookup fields to filter data."""
+
     list_order_by = ()
+    """List of fields or lookup fields to order data."""
+
     list_group_by = ()
+    """List of fields or lookup fields to group data."""
+
     list_serie_fields = ()
+    """List of fields to group by results in chart."""
+
     template_name = None
+    """Template file name to render the report."""
+
     title = None
+    """Title of the report."""
+
     type = 'report'
+    """"report" for only report and  "chart" for report and chart graphic results."""
+
     group_totals = {}
+    """Dictionary with field name as key and function to calculate their values."""
+
     report_totals = {}
+    """Dictionary with field name as key and function to calculate their values."""
+
     override_field_values = {}
+    """
+    Dictionary with field name as key and function to parse their original values.
+
+    ::
+
+        override_field_values = {
+            'men': men_format,
+            'women': women_format
+        }
+    """
     override_field_formats = {}
+    """Dictionary with field name as key and function to parse their value after :func:`override_field_values`."""
+
     override_field_labels = {}
+    """Dictionary with field name as key and function to parse the column label."""
+
     override_field_choices = {}
+    """#TODO"""
+
     override_field_filter_values = {}
+    """#TODO"""
+
     override_group_value = {}
+    """#TODO"""
+
     chart_types = ()
+    """List of highchart types."""
+
     exports = ('excel', 'pdf')
+    """Alternative render report as "pdf" or "csv"."""
+
     inlines = []
+    """List of other's Report related to the main report."""
+
     query_set = None
+    """#TODO"""
 
     def __init__(self, parent_report=None, request=None):
         self.parent_report = parent_report
@@ -205,7 +265,7 @@ class ReportAdmin(object):
             pass
         return value
 
-    @cache_return
+    # @cache_return
     def get_m2m_field_names(self):
         return [field for ffield, field, index, mfield in self.model_m2m_fields]
 
@@ -232,6 +292,9 @@ class ReportAdmin(object):
         return [x for x in self.fields if not x in self.related_fields]
 
     def get_column_names(self, ignore_columns={}):
+        """
+        Return the list of columns
+        """
         values = []
         for field, field_name in self.model_fields:
             if field_name in ignore_columns:
@@ -240,7 +303,7 @@ class ReportAdmin(object):
             values.append(caption)
         return values
 
-    @cache_return
+    # @cache_return
     def get_query_field_names(self):
         values = []
         for field in self.get_fields():
@@ -250,8 +313,11 @@ class ReportAdmin(object):
                 values.append(field)
         return values
 
-    @cache_return
+    # @cache_return
     def get_query_set(self, filter_kwargs):
+        """
+        Return the the queryset
+        """
         qs = self.model.objects.all()
         for k, v in filter_kwargs.items():
             if not v is None and v != '':
@@ -263,6 +329,9 @@ class ReportAdmin(object):
         return self.query_set
 
     def get_title(self):
+        """
+        Return the report title
+        """
         title = self.title or None
         if not title:
             if not self.model:
@@ -391,6 +460,7 @@ class ReportAdmin(object):
             return context
         finally:
             globals()['_cache_class'] = {}
+            
 
     def render(self, request, extra_context={}):
         context_or_response = self.get_render_context(request, extra_context)
@@ -465,15 +535,15 @@ class ReportAdmin(object):
 
         return form
 
-    @cache_return
+    # @cache_return
     def get_groupby_fields(self):
         return [(mfield, field, caption) for (mfield, field), caption in zip(self.model_fields, self.get_column_names()) if field in self.list_group_by]
 
-    @cache_return
+    # @cache_return
     def get_serie_fields(self):
         return [(index, mfield, field, caption) for index, ((mfield, field), caption) in enumerate(zip(self.model_fields, self.get_column_names())) if field in self.list_serie_fields]
 
-    @cache_return
+    # @cache_return
     def get_form_groupby(self, request):
         groupby_fields = self.get_groupby_fields()
 
@@ -801,7 +871,7 @@ class ReportAdmin(object):
 
             # gqs_values needs to already be sorted on the same key function
             # for groupby to work properly
-            gqs_values = sorted(gqs_values, key=get_key_values)
+            gqs_values.sort(key=get_key_values)
             res = groupby(gqs_values, key=get_key_values)
             row_values = {}
             for key, values in res:
@@ -819,15 +889,19 @@ class ReportAdmin(object):
         if self.model_m2m_fields:
             qs_list = group_m2m_field_values(qs_list)
 
+        groupby_fn = None
         if groupby_data and groupby_data['groupby']:
             groupby_field = groupby_data['groupby']
             if groupby_field in self.override_group_value:
                 transform_fn = self.override_group_value.get(groupby_field)
-                g = groupby(qs_list, lambda x: transform_fn(x[ffields.index(groupby_field)]))
+                groupby_fn = lambda x: transform_fn(x[ffields.index(groupby_field)])
             else:
-                g = groupby(qs_list, lambda x: x[ffields.index(groupby_field)])
+                groupby_fn = lambda x: x[ffields.index(groupby_field)]
         else:
-            g = groupby(qs_list, lambda x: None)
+            groupby_fn = lambda x: None
+
+        qs_list.sort(key=groupby_fn)
+        g = groupby(qs_list, key=groupby_fn)
 
         row_report_totals = self.get_empty_row_asdict(self.report_totals, [])
         for grouper, resources in g:
